/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: ui.c
 * Date: 02.12.2013
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Implements user interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#include "ui.h"

#include <stdio.h>
#include <xparameters.h>


/*
 *  Returns recieved char
 */
unsigned char ui_getch(void)
{
    return (unsigned char)UART_RecvByte(STDIN);
}


/*
 *  Send a char
 */
void ui_putch(unsigned char ch)
{
    UART_sendByte(STDOUT, ch);
}


/*
 * non-blocking reading STDIN (UART).
 * If there is no new byte the function returns -1
 */
#if defined __MICROBLAZE__
unsigned char ui_getch_nb(void)
{
    unsigned char ch = UART_GetStatusReg(STDIN);
    if (ch & UART_SR_RX_FIFO_VALID_DATA)
        return UART_RecvByte(STDIN);
    else
        return 0xff;
}
#else
unsigned char ui_getch_nb(void)
{
    unsigned char ch = 0xff;
    if (UART_IsReceiveData(STDIN))
        while ( UART_IsReceiveData(STDIN) ) {
            ch = UART_RecvByte(STDIN);
        }
    return ch;
}
#endif

/*
 *  waits for a digit
 */
int ui_get_digit(unsigned int base)
{
    int ch;

    // one digit can be only 0-9 and 'a'-'z'
    if (base > ('z' - 'a' + 11))
        return -1;

    ch = ui_getch();

    if (base < 11) {
        if (ch >= '0' && ch < ('0' + base))
            return (ch-'0');
        return -ch;
    } else {
        if (ch >= '0' && ch <= '9')
            return (ch-'0');
        if (ch >= 'a' && ch <= ('a' + base - 11))
            return (ch - 'a' + 10);
        if (ch >= 'A' && ch <= ('A' + base - 11))
            return (ch-'A'+10);
        return -ch;
    }
}

/*
 *  get a digit - nonblocking
 */
int ui_get_digit_nb(unsigned int base)
{
    int ch;

    // one digit can be only 0-9 and 'a'-'z'
    if (base > ('z' - 'a' + 11))
      return -1;

    ch = ui_getch_nb();

    if (base < 11) {
        if (ch >= '0' && ch < ('0' + base))
            return (ch-'0');
        return NO_KEY;
    } else {
        if (ch >= '0' && ch <= '9')
            return (ch-'0');
        if (ch >= 'a' && ch <= ('a' + base - 11))
            return (ch - 'a' + 10);
        if (ch >= 'A' && ch <= ('A' + base - 11))
            return (ch-'A'+10);
        return NO_KEY;
    }
}


/*
 * send a digit
 */
void ui_put_digit(int dig)
{
    if (dig < 0) {
        ui_putch(-dig);
        return;
    }
    if (dig < 10) {
        ui_putch(dig + '0');
        return;
    }
    ui_putch(dig + 'A' - 10);
}


/*
 * get a value
 */
int ui_get_value(const char *text)
{
    unsigned char val;
    unsigned char neg;
    int ch;

    if (text) printf("\r\n%s ", text);

    val = 0;
    neg = 0;

    do {
        ch = ui_get_digit(16);
        if ((ch < 0) && ((-ch) == '-')) {
            neg = !neg;
            printf("-");
        }
    } while (ch < 0);

    ui_put_digit(ch);
    val = (ch << 4) & 0xf0;

    do {
        ch = ui_get_digit(16);
    } while (ch < 0);

    ui_put_digit(ch);
    val |= (ch & 0x0f);

    if (neg)
      return (((int)val) * (-1));
    else
      return val;
}

/*
 *  get an address
 */
int ui_get_address(int bits)
{
    int i;
    int ch;
    unsigned long addr = 0;

    i = bits/4;

    while(i-- > 0) {
        do {
            ch = ui_get_digit(16);
        } while (ch < 0);
        ui_put_digit(ch);
        addr = (addr << 4) | (ch & 0x0f);
    }
    return addr;
}

/*
 * get an integer value
 */
int ui_get_integer(int digs)
{
    int ch;
    int val = 0;
    int dec = 0;
    int neg = 0;

    do {
        ch = ui_get_digit(10);
        if (ch < 0) {
            if (dec == 0 && ch == -(int)'-') {
                neg = 1;
                ui_put_digit(ch);
            } else if (ch == KEY_ENTER || ch == KEY_ESC) {
                break;
            }
        } else {
            if (dec<digs) {
                ui_put_digit(ch);
                val = val * 10 + ch;
                dec++;
            }
        }
    } while(1);

    ui_putch(10);
    ui_putch(13);

    if (ch == -13) {
        if (neg) val = -val;
        return val;
    } else
        return -1;

}

int ui_get_float(int decdig, float *retval)
{
    int ch;
    float val = 0;
    float mul = 0.1;
    int dec = 0;
    int neg = 0;
    int dg = 0;

    do {
        ch = ui_get_digit(10);

        // wait for dec.point or ENTER
        if (ch < 0) {
            if (dec == 0) {
                // dec.point
                if (ch == -((int)'.') || ch == -((int)',')) {
                    dec = 1;
                    ui_put_digit(ch);
                }
            }
            if (val == 0 && neg == 0) {
                // negative value
                if (ch == -((int)'-')) {
                    neg = 1;
                    ui_put_digit(ch);
                }
            }
            // ENTER or ESC
            if (ch == KEY_ENTER || ch == KEY_ESC) {
                break;
            }
        } else {
            if (dec) {
                if (decdig) {
                  ui_put_digit(ch);
                  val = val + (mul * ch);
                  mul /= 10;
                  decdig--;
                  dg++;
                }
            } else {
                ui_put_digit(ch);
                val = (val * 10) + ch;
                dg++;
            }
        }
    } while (1);

    ui_putch(10);
    ui_putch(13);

    if (ch == -13) {
        // there is no new value
        if (dg == 0) {
            return 1;
        } else {
            if (neg) val = -val;
            if (retval) *retval = val;
            return 0;
        }
    } else {
        return -1;
    }
}

/*
 * conversion function
 */

// for 64 digits
static char ui_conversion_buffer[65];

char *ui_conv_to_bin(unsigned long val, int bits)
{
    int i = 0;
    unsigned long mask = 1 << (bits - 1);

    while (bits--) {
        ui_conversion_buffer[i++] = (val & mask) ? '1' : '0';
        mask >>= 1;
    }

    ui_conversion_buffer[i] = 0;
    return (char *) &ui_conversion_buffer[0];
}


/*
 * print menu
 */
void ui_print_menu(const char *name, const ui_menu_t menu[], unsigned char *print_en)
{
    int i = 0;

    if (*print_en) {
        printf("\n%s:\n", name);
        printf("------------------------------------------------\n");

        while (menu[i].key) {
            printf("%c - %s\r\n", menu[i].key, menu[i].item);
            i++;
        }

        printf("------------------------------------------------\n\n");

        *print_en = 0;
    }
}



